# Copyright 2019 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys, os, subprocess, glob

script_dir = os.path.dirname(os.path.realpath(__file__))

publisher_command = os.environ.get("SIMPLE_COMMUNICATION_PUBLISHER_BIN")
if not publisher_command:
    publisher_files = glob.glob(os.path.join(script_dir, "**/SimpleCommunicationPublisher*"), recursive=True)
    publisher_command = next(iter(publisher_files), None)
assert publisher_command
subscriber_command = os.environ.get("SIMPLE_COMMUNICATION_SUBSCRIBER_BIN")
if not subscriber_command:
    subscriber_files = glob.glob(os.path.join(script_dir, "**/SimpleCommunicationSubscriber*"), recursive=True)
    subscriber_command = next(iter(subscriber_files), None)
assert subscriber_command
xml_file = os.environ.get("XML_FILE")
if xml_file:
    real_xml_file = os.path.join(script_dir, xml_file)
else:
    real_xml_file = os.path.join(script_dir, "liveliness_assertion.360.xml")

subscriber_proc = subprocess.Popen([subscriber_command, "--seed", str(os.getpid()), "--notexit",
    "--xmlfile", real_xml_file], stdout=subprocess.PIPE)
publisher_proc = subprocess.Popen([publisher_command, "--seed", str(os.getpid()), "--exit_on_lost_liveliness",
    "--xmlfile", real_xml_file])

while True:
    line = subscriber_proc.stdout.readline()
    if line.strip().decode('utf-8') == 'Publisher recovered liveliness':
        print("Publisher recovered liveliness")
        break

publisher_proc.kill()
subscriber_proc.communicate()
retvalue = subscriber_proc.returncode

if retvalue != 0:
    print("Test failed: " + str(retvalue))
else:
    print("Test successed")

sys.exit(retvalue)



# Copyright 2020 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
    Tests for the fastdds tool.

    Contains a package of system test for fastdds tool

    usage: test.py <install_path> <test_name>

    install_path: Fast-DDS base path installation

    test_name: Test to run.

    Available tests:

        test_fastdds_installed
        test_fastdds_discovery
        test_fastdds_shm

"""

import argparse
import os
import subprocess
import sys
from pathlib import Path


def setup_script_name():
    """
    Test script for POSIX os.

    Return the name of the setup script file. This setup script is
    required by some tests to prepare the test environment

    """
    if os.name == 'posix':
        script_name = 'setup.bash'
    elif os.name == 'nt':
        script_name = 'setup.bat'
    else:
        print(f'{os.name} not supported')
        sys.exit(1)

    return script_name


def cmd(install_path, setup_script_path=Path(), args=''):
    """
    Build the command line to run for the current platform.

    install_path Path:
        Fast-DDS instalation path.

    setup_script_path Path:
        Path to the setup script if necessary.

    args str:
        Extra argumens for the command (sub-verb)

    """
    tool_path = str(install_path.resolve())
    if os.name == 'posix':
        if str(setup_script_path) == '.':
            cmd = f'bash -c "\\"{tool_path}\\" {args}"'
        else:
            cmd = (f'bash -c ". \\"{setup_script_path}\\" && \\"{tool_path}\\"'
                   f' {args}"')
    elif os.name == 'nt':
        if str(setup_script_path) == '.':
            cmd = f'"{tool_path}" {args}'
        else:
            cmd = f'"{setup_script_path}" && "{tool_path}" {args}'
    else:
        print(f'{os.name} not supported')
        sys.exit(1)
    return cmd


def test_fastdds_installed(install_path):
    """Test that fastdds is installed and runs."""
    ret = subprocess.call(cmd(install_path), shell=True)
    if 0 != ret:
        print('test_fastdds_installed FAILED')
        sys.exit(ret)


def test_fastdds_shm(install_path):
    """Test that shm command runs."""
    args = ' shm clean'
    ret = subprocess.call(cmd(
        install_path=install_path, args=args), shell=True)
    if 0 != ret:
        print('test_fastdds_shm FAILED')
        sys.exit(ret)


def test_fastdds_discovery(install_path, setup_script_path):
    """Test that discovery command runs."""
    args = ' discovery'
    ret = subprocess.call(
        cmd(install_path=install_path,
            setup_script_path=setup_script_path,
            args=args),
        shell=True)

    if 0 != ret:
        print('test_fastdds_discovery FAILED')
        sys.exit(ret)

def test_ros_discovery(install_path, setup_script_path):
    """Test that discovery command runs."""
    ret = subprocess.call(
        cmd(install_path=install_path,
            setup_script_path=setup_script_path),
        shell=True)

    if 0 != ret:
        print('test_fastdds_discovery FAILED')
        sys.exit(ret)

def get_paths(install_path):
    """
    Adjust the install path when --merge-install has been used.

    param install_path Path:
        Path to the Fast-DDS installation path

    return Path:
        Adjusted path to the installation path where fastdds tool
        is installed

    """
    tool_install_path = install_path / 'bin'

    if not os.path.exists(tool_install_path.resolve()):
        tool_install_path = tool_install_path / '..' / 'bin'
        if not os.path.exists(tool_install_path.resolve()):
            print(f'{tool_install_path} NOT FOUND')
            sys.exit(1)

    setup_script_path = install_path / setup_script_name()
    if not os.path.exists(str(setup_script_path.resolve())):
        setup_script_path = install_path / '..' / setup_script_name()
        if not os.path.exists(str(setup_script_path.resolve())):
            print(f'setup_script NOT FOUND')
            sys.exit(1)

    return setup_script_path, tool_install_path


if __name__ == '__main__':

    parser = argparse.ArgumentParser(
            usage='test.py <install_path> <test_name>',
        )

    parser.add_argument('install_path',
                        help='FastDDS install path')

    parser.add_argument('test_name',
                        help='Test to run')

    args = parser.parse_args()

    setup_script_path, tool_path = get_paths(Path(args.install_path))
    
    fastdds_tool_path = tool_path / 'fastdds'
    ros_disc_tool_path = tool_path / 'ros-discovery'

    # Tests dictionary
    tests = {
        'test_fastdds_installed': lambda: test_fastdds_installed(fastdds_tool_path),
        'test_fastdds_discovery': lambda: test_fastdds_discovery(
            fastdds_tool_path, setup_script_path),
        'test_ros_discovery': lambda: test_ros_discovery(ros_disc_tool_path,
            setup_script_path),
        'test_fastdds_shm': lambda: test_fastdds_shm(fastdds_tool_path)
    }

    tests[args.test_name]()



# Copyright 2020 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sqlite3
import sys

"""
Return the number of apparitions of a guid, on a specific table and database.

It takes three arguments:
    * database_name: The filename of the database
    * table_name: The name of the table within the database
    * check_guid: The guid of the entity that you want to look for in the
      stated table and database.

"""

if __name__ == '__main__':
    if len(sys.argv) != 4:
        sys.exit(255)
    database_name = str(sys.argv[1])
    table_name = str(sys.argv[2])
    check_guid = str(sys.argv[3])

    command = f"SELECT guid FROM {table_name} WHERE guid='{check_guid}'"
    try:
        connection = sqlite3.connect(database_name)
        c = connection.cursor()
        c.execute(command)
        num = len(c.fetchall())
        connection.close()
    except sqlite3.DatabaseError as e:
        sys.exit(255)

    sys.exit(num)



# Copyright 2020 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
    fastdds shm sub-command.

    This verb provides maintenance tasks related with Fast-DDS shared-memory
    transport.

    usage: fastdds shm [<shm-command>]

    shm-commands:

        clean     clean SHM zombie files

    positional arguments:
        command     shm-command to run

    optional arguments:
        -h, --help  show this help message and exit

"""

import argparse

from shm.clean import Clean


class Parser:
    """Shared-memory sub-commands parser."""

    __help_message = """fastdds shm [<shm-command>]\n\n
    shm-commands:\n\n
    \tclean     clean SHM zombie files
    """

    def __init__(self, argv):
        """Parse the sub-command and dispatch to the appropriate handler.

        Shows usage if no sub-command is specified.

        Supported sub-commands:

            clean   clean SHM zombie files

        param argv list(str):
            list containing the arguments for the command
        """
        parser = argparse.ArgumentParser(
            usage=self.__help_message,
            add_help=True
        )

        parser.add_argument('command', nargs='?', help='shm-command to run')

        args = parser.parse_args(argv)

        if args.command is not None:
            if args.command == 'clean':
                Clean().run()
            else:
                print('shm-command ' + args.shm_command + ' is not valid')
        else:
            parser.print_help()



# Copyright 2021 Proyectos y Sistemas de Mantenimiento SL (eProsima)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import glob
import os
import queue
import subprocess
import sys
import threading
import time

script_dir = os.path.dirname(os.path.realpath(__file__))

# Test executable
process_command = os.environ.get(
    'CLIENT_SERVER_DYNAMIC_DISCOVERY_BIN')
if not process_command:
    process_files = glob.glob(
        os.path.join(
           script_dir,
           '**/DDSParticipantDiscovery*'),
        recursive=True)
    pf = iter(process_files)
    process_command = next(pf, None)
    while process_command and \
        (not os.path.isfile(process_command)
         or not os.access(process_command,
         os.X_OK)):
        process_command = next(pf, None)
assert(process_command)

# Thread that read process output and push it into a queue
def output_reader(proc, outq):
    for line in iter(proc.stdout.readline, b''):
        outq.put(line.decode('utf-8'))

def first_step(outq):
    first_step_fulfilled = False
    server_1_discover_client = False
    count = 0
    initial_time = time.time()
    while not first_step_fulfilled:
        global stop_threads
        if stop_threads:
            break
        try:
            line = outq.get(block=False).rstrip()
            print(line)
            sys.stdout.flush()

            assert '44.53.00.5f.45.50.52.4f.53.49.4d.41' in line
            assert 'discovered participant' in line
            count = count + 1
            if 'discovered participant 44.53.00.5f.45.50.52.4f.53.49.4d.41|0.0.1.c1: 1' in line:
                print('CLIENT OVERRIDE discovered SERVER 1')
                server_1_discover_client = True
        except queue.Empty:
            # Ensure that 2 s has passed so the file watch can detect that the file has changed
            if server_1_discover_client and count >= 2 and (time.time() - initial_time) > 2:
                if count == 2:
                    first_step_fulfilled = True
                else:
                    print('ERROR: More discoveries than expected')
                    stop_threads = True
                    sys.exit(1)
            sys.stdout.flush()
        except AssertionError:
            print('ASSERTION ERROR: ' + line)
            stop_threads = True
            sys.exit(1)
        time.sleep(0.1)

def second_step(outq):
    second_step_fulfilled = False
    server_2_discover_client = False
    client_2_warning = False
    count = 0
    initial_time = time.time()
    while not second_step_fulfilled:
        global stop_threads
        if stop_threads:
            break
        try:
            line = outq.get(block=False).rstrip()
            print(line)
            sys.stdout.flush()

            if 'discovered participant' in line:
                assert '44.53.01.5f.45.50.52.4f.53.49.4d.41' in line
                count = count + 1
                if 'discovered participant 44.53.01.5f.45.50.52.4f.53.49.4d.41|0.0.1.c1: 2' in line:
                    print('CLIENT OVERRIDE discovered SERVER 2')
                    server_2_discover_client = True
            elif 'Warning' in line:
                # Client 2 does not discover anyone
                assert 'Trying to add Discovery Servers to a participant which is not a SERVER, BACKUP or an' in line
                assert 'overriden CLIENT (SIMPLE participant transformed into CLIENT with the environment variable)' in line
                client_2_warning = True
            else:
                assert 'detected changes on participant' in line
        except queue.Empty:
            # Ensure that 2 s has passed so the file watch can detect that the file has changed
            if server_2_discover_client and client_2_warning and count >= 2 and (time.time() - initial_time) > 2:
                if count == 2:
                    second_step_fulfilled = True
                else:
                    print('ERROR: More discoveries than expected')
                    stop_threads = True
                    sys.exit(1)
            sys.stdout.flush()
        except AssertionError:
            print('ASSERTION ERROR: ' + line)
            stop_threads = True
            sys.exit(1)
        time.sleep(0.1)

def third_step(outq):
    third_step_fulfilled = False
    server_1_discover_server_2 = False
    server_2_discover_server_1 = False
    initial_time = time.time()
    while not third_step_fulfilled:
        global stop_threads
        if stop_threads:
            break
        try:
            line = outq.get(block=False).rstrip()
            print(line)
            sys.stdout.flush()

            if 'Participant 44.53.01.5f.45.50.52.4f.53.49.4d.41|0.0.1.c1 discovered participant' in line and \
                '44.53.00.5f.45.50.52.4f.53.49.4d.41|0.0.1.c1: 2' in line:
                print ('SERVER 2 discovers SERVER 1')
                server_2_discover_server_1 = True
            elif 'Participant 44.53.00.5f.45.50.52.4f.53.49.4d.41|0.0.1.c1 discovered participant' in line and \
                '44.53.01.5f.45.50.52.4f.53.49.4d.41|0.0.1.c1: 2' in line:
                print ('SERVER 1 discovers SERVER 2')
                server_1_discover_server_2 = True
            else:
                assert 'detected changes on participant' in line
        except queue.Empty:
            # Ensure that 2 s has passed so the file watch can detect that the file has changed
            if server_1_discover_server_2 and server_2_discover_server_1 and (time.time() - initial_time) > 2:
                third_step_fulfilled = True
            sys.stdout.flush()
        except AssertionError:
            print('ASSERTION ERROR: ' + line)
            stop_threads = True
            sys.exit(1)
        time.sleep(0.1)

def fourth_step(outq):
    fourth_step_fulfilled = False
    warning_client_1 = False
    warning_client_2 = False
    count = 0
    initial_time = time.time()
    while not fourth_step_fulfilled:
        global stop_threads
        if stop_threads:
            break
        try:
            line = outq.get(block=False).rstrip()
            print(line)
            sys.stdout.flush()

            if 'Trying to add Discovery Servers to a participant which is not a SERVER, BACKUP or an' in line \
                and 'overriden CLIENT (SIMPLE participant transformed into CLIENT with the environment variable)' in line:
                warning_client_2 = True
            elif 'Discovery Servers cannot add/modify their locators' in line:
                warning_client_1 = True
            elif 'discovered participant' in line:
                count = count + 1
        except queue.Empty:
            # Ensure that 2 s has passed so the file watch can detect that the file has changed
            if warning_client_1 and warning_client_2 and count == 0 and (time.time() - initial_time) > 2:
                fourth_step_fulfilled = True
            elif count > 0:
                print('ERROR: More discoveries than expected')
                stop_threads = True
                sys.exit(1)
            sys.stdout.flush()
        except AssertionError:
            print('ASSERTION ERROR: ' + line)
            stop_threads = True
            sys.exit(1)
        time.sleep(0.1)

def fifth_step(outq):
    fifth_step_fulfilled = False
    warning = False
    count = 0
    initial_time = time.time()
    while not fifth_step_fulfilled:
        global stop_threads
        if stop_threads:
            break
        try:
            line = outq.get(block=False).rstrip()
            print(line)
            sys.stdout.flush()

            if 'Discovery Servers cannot add/modify their locators' in line:
                warning = True
            elif 'discovered participant' in line:
                count = count + 1
        except queue.Empty:
            # Ensure that 2 s has passed so the file watch can detect that the file has changed
            if warning and count == 0 and (time.time() - initial_time) > 2:
                fifth_step_fulfilled = True
            elif count > 0:
                print('ERROR: More discoveries than expected')
                stop_threads = True
                sys.exit(1)
            sys.stdout.flush()
        except AssertionError:
            print('ASSERTION ERROR: ' + line)
            stop_threads = True
            sys.exit(1)
        time.sleep(0.1)

def sixth_step(outq):
    sixth_step_fulfilled = False
    warning_client_1 = False
    warning_client_2 = False
    count = 0
    initial_time = time.time()
    while not sixth_step_fulfilled:
        global stop_threads
        if stop_threads:
            break
        try:
            line = outq.get(block=False).rstrip()
            print(line)
            sys.stdout.flush()

            if 'Trying to add Discovery Servers to a participant which is not a SERVER, BACKUP or an' in line \
                and 'overriden CLIENT (SIMPLE participant transformed into CLIENT with the environment variable)' in line:
                warning_client_2 = True
            elif 'Discovery Servers cannot be removed from the list; they can only be added' in line:
                warning_client_1 = True
            elif 'discovered participant' in line:
                count = count + 1
        except queue.Empty:
            # Ensure that 2 s has passed so the file watch can detect that the file has changed
            if warning_client_1 and warning_client_2 and count == 0 and (time.time() - initial_time) > 2:
                sixth_step_fulfilled = True
            elif count > 0:
                print('ERROR: More discoveries than expected')
                stop_threads = True
                sys.exit(1)
            sys.stdout.flush()
        except AssertionError:
            print('ASSERTION ERROR: ' + line)
            stop_threads = True
            sys.exit(1)
        time.sleep(0.1)

def seventh_step(outq):
    seventh_step_fulfilled = False
    warning = False
    count = 0
    initial_time = time.time()
    while not seventh_step_fulfilled:
        global stop_threads
        if stop_threads:
            break
        try:
            line = outq.get(block=False).rstrip()
            print(line)
            sys.stdout.flush()

            if 'Discovery Servers cannot be removed from the list; they can only be added' in line:
                warning = True
            elif 'discovered participant' in line:
                count = count + 1
        except queue.Empty:
            # Ensure that 2 s has passed so the file watch can detect that the file has changed
            if warning and count == 0 and (time.time() - initial_time) > 2:
                seventh_step_fulfilled = True
            elif count > 0:
                print('ERROR: More discoveries than expected')
                stop_threads = True
                sys.exit(1)
            sys.stdout.flush()
        except AssertionError:
            print('ASSERTION ERROR: ' + line)
            stop_threads = True
            sys.exit(1)
        time.sleep(0.1)

def exit(cv):
    cv.release()
    print('ERROR: timeout without expected discovery happening')
    global stop_threads
    stop_threads = True
    os.remove(server_1_env_file)
    os.remove(server_2_env_file)
    os.remove(client_env_file)
    sys.exit(1)

def communication(proc, outq, outt, cv):
    """A"""
    t = threading.Thread(target=output_reader, args=(proc,outq))
    t.start()

    try:
        time.sleep(0.2)

        while True:
            global stop_threads
            if stop_threads:
                break
            try:
                line = outt.get(block=False).rstrip()
                print(line)
                sys.stdout.flush()

                if "FIRST STEP" in line:
                    first_step(outq)
                    cv.acquire()
                    cv.notify()
                    cv.release()
                elif "SECOND STEP" in line:
                    second_step(outq)
                    cv.acquire()
                    cv.notify()
                    cv.release()
                elif "THIRD STEP" in line:
                    third_step(outq)
                    cv.acquire()
                    cv.notify()
                    cv.release()
                elif "FOURTH STEP" in line:
                    fourth_step(outq)
                    cv.acquire()
                    cv.notify()
                    cv.release()
                elif "FIFTH STEP" in line:
                    fifth_step(outq)
                    cv.acquire()
                    cv.notify()
                    cv.release()
                elif "SIXTH STEP" in line:
                    sixth_step(outq)
                    cv.acquire()
                    cv.notify()
                    cv.release()
                elif "SEVENTH STEP" in line:
                    seventh_step(outq)
                    cv.acquire()
                    cv.notify()
                    cv.release()
                
 
            except queue.Empty:
                sys.stdout.flush()
            
            time.sleep(0.1)
    finally:
        proc.terminate()

    while outq.empty() != True:
        line = outq.get(block=False).rstrip()
        print(line)

    t.join()

# Random unicast port
random_port_server_1 = os.environ.get(
    'W_UNICAST_PORT_RANDOM_NUMBER')
random_port_server_2 = str(int(random_port_server_1) + 1)

# Condition variable
cv = threading.Condition()

# Environment files
server_1_env_file = "server_1_env_file.json"
server_2_env_file = "server_2_env_file.json"
client_env_file = "client_env_file.json"
# Both server environment files are created empty
open(server_1_env_file, 'w+').close()
open(server_2_env_file, 'w+').close()
# Client environment file should include the locator for the first server
f = open(client_env_file, 'w+')
f.write('{"ROS_DISCOVERY_SERVER": "localhost:')
f.write(random_port_server_1)
f.write('"}')
f.close()

outq = queue.Queue()
outt = queue.Queue()
outt.put("TEST RUNNING\n")
outt.put("FIRST STEP: Override Client discovers Server 1\n")

server_1_process = subprocess.Popen([process_command,
    "--discovery_protocol", "SERVER",
    "--guid_prefix", "44.53.00.5F.45.50.52.4F.53.49.4D.41",
    "--unicast_metatraffic_locator", random_port_server_1],
    env={"FASTDDS_ENVIRONMENT_FILE": server_1_env_file},
    stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
server_2_process = subprocess.Popen([process_command,
    "--discovery_protocol", "SERVER",
    "--guid_prefix", "44.53.01.5F.45.50.52.4F.53.49.4D.41",
    "--unicast_metatraffic_locator", random_port_server_2],
    env={"FASTDDS_ENVIRONMENT_FILE": server_2_env_file},
    stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
# The client must be DiscoveryProtocol::SIMPLE to use the environment variable
client_override_process = subprocess.Popen(process_command,
    env={"FASTDDS_ENVIRONMENT_FILE": client_env_file},
    stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
# DiscoveryProtocol::CLIENT, environment variable does not apply either initializing as updating
client_process = subprocess.Popen([process_command,
    "--discovery_protocol", "CLIENT"],
    env={"FASTDDS_ENVIRONMENT_FILE": client_env_file},
    stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

stop_threads = False
t_0 = threading.Thread(target=communication, args=(server_1_process,outq,outt,cv))
t_1 = threading.Thread(target=communication, args=(server_2_process,outq,outt,cv))
t_2 = threading.Thread(target=communication, args=(client_override_process,outq,outt,cv))
t_3 = threading.Thread(target=communication, args=(client_process,outq,outt,cv))

t_0.start()
t_1.start()
t_2.start()
t_3.start()

# Wait 10 seconds for the condition variable to be notified
cv.acquire()
result = cv.wait(10)
if result == False:
    exit(cv)

outt.put("SECOND STEP: Override Client discovers both Servers. Normal Client does not discover anyone\n")

# Add second server to client
f = open(client_env_file, 'w+')
f.write('{"ROS_DISCOVERY_SERVER": "localhost:')
f.write(random_port_server_1)
f.write(';localhost:')
f.write(random_port_server_2)
f.write('"}')
f.close()

result = cv.wait(10)
if result == False:
    exit(cv)

outt.put("THIRD STEP: Both Servers discover each other\n")

# Add second server to first server
f = open(server_1_env_file, 'w+')
f.write('{"ROS_DISCOVERY_SERVER": ";localhost:')
f.write(random_port_server_2)
f.write('"}')
f.close()

result = cv.wait(10)
if result == False:
    exit(cv)

outt.put("FOURTH STEP: Changing a Server locator from the Client list outputs a Log Warning\n")

# Change first server locator from client list
f = open(client_env_file, 'w+')
f.write('{"ROS_DISCOVERY_SERVER": "localhost:11811;localhost:')
f.write(random_port_server_2)
f.write('"}')
f.close()

result = cv.wait(10)
if result == False:
    exit(cv)

outt.put("FIFTH STEP: Changing a Server locator from the Server list outputs a Log Warning\n")

# Change server locator from server list
f = open(server_1_env_file, 'w+')
f.write('{"ROS_DISCOVERY_SERVER": ";localhost:11811"}')
f.close()

result = cv.wait(10)
if result == False:
    exit(cv)

outt.put("SIXTH STEP: Removing a Server from the Client list outputs a Log Warning\n")

# Remove first server from client list
f = open(client_env_file, 'w+')
f.write('{"ROS_DISCOVERY_SERVER": ";localhost:')
f.write(random_port_server_2)
f.write('"}')
f.close()

result = cv.wait(10)
if result == False:
    exit(cv)

outt.put("SEVENTH STEP: Removing a Server from the  Server list outputs a Log Warning\n")

# Remove server from the server 1 list
f = open(server_1_env_file, 'w+')
f.write('{"ROS_DISCOVERY_SERVER": ""}')
f.close()

result = cv.wait(10)
if result == False:
    exit(cv)

outt.put("Killing processes\n")

cv.release()
# Kill processes
stop_threads = True

t_0.join()
t_1.join()
t_2.join()
t_3.join()

# Delete files
os.remove(server_1_env_file)
os.remove(server_2_env_file)
os.remove(client_env_file)



# Copyright 2020 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
    Tests for the fastdds tool.

    Contains a package of system test for fastdds tool

    usage: test.py <binary_path> <test_name>

    binary_path: Fast-DDS binary path

    test_name: Test to run.

    Available tests:

        test_fast_discovery_closure

"""

import argparse
import subprocess
import sys
import time
import signal
import os

def signal_handler(signum, frame):
    # ignore signals if the test generates them
    pass

def test_fast_discovery_closure(fast_discovery_tool):
    """Test that discovery command closes correctly."""
    command = [fast_discovery_tool, '-i', '0']

    print("Executing command: " + str(command))

    # this subprocess cannot be executed in shell=True or using bash
    #  because a background script will not broadcast the signals
    #  it receives
    proc = subprocess.Popen(command,
                            stdout=subprocess.PIPE,
                            universal_newlines=True
                           )

    # sleep to let the server run
    time.sleep(1)

    # 1. An exit code of 0 means everything was alright
    # 2. An exit code of 1 means the tool's process terminated before even
    #    sending the kill signal.
    # 3. An exit code of 2 means the signal could not terminate the process
    # 4. An exit code of 3 means the signal terminated the process, but the
    #    output was different than expected
    exit_code = 0

    # Check whether the process has terminated already
    if not proc.poll() is None:
        # If the process has already exit means something has gone wrong.
        # Capture and print output for traceability and exit with code s1.
        output, err = proc.communicate()
        print('test_fast_discovery_closure FAILED on launching tool')
        print('STDOUT:')
        print(output)
        print('STDERR:')
        print(err)
        sys.exit(1)

    # direct this script to ignore SIGINT
    signal.signal(signal.SIGINT, signal_handler)

    # send SIGINT to process and wait for processing
    lease = 0
    while True:

        if os.name == 'posix':
            proc.send_signal(signal.SIGINT)
        elif os.name == 'nt':
            proc.send_signal(signal.CTRL_C_EVENT)

        time.sleep(1)
        lease += 1

        # Break when signal kills the process or it hangs
        if proc.poll() is None and lease < 10:
            print('iterating...')
        else:
            break

    # Check whether SIGINT was able to terminate the process
    if proc.poll() is None:
        # SIGINT couldn't terminate the process. Kill it and exit with code 2
        proc.kill()
        print('Signal could not kill process')
        sys.exit(2)

    # Get process output
    output, err = proc.communicate()

    EXPECTED_CLOSURE = "### Server shut down ###"

    # Check whether everything closed down nicely
    if EXPECTED_CLOSURE in output:
        # Success
        print('test_fast_discovery_closure SUCCEED')
    else:
        # Failure
        print('test_fast_discovery_closure FAILED')
        print('STDOUT:')
        print(output)
        print('STDERR:')
        print(err)
        exit_code = 3

    sys.exit(exit_code)

if __name__ == '__main__':

    parser = argparse.ArgumentParser(
            usage='test.py <binary_path> <test_name>',
        )

    parser.add_argument('binary_path',
                        help='''fast-discovery-server binary fully
                        qualified path''')

    parser.add_argument('test_name',
                        help='Test to run')

    args = parser.parse_args()

    # Tests dictionary
    tests = {
        'test_fast_discovery_closure': lambda: test_fast_discovery_closure(
            args.binary_path),
    }

    tests[args.test_name]()



import matplotlib.pyplot as plt
from matplotlib import style
import numpy as np
import pandas as pd
import sys

if len(sys.argv) < 2:
    print("Bad usage.")
    print("    allocation_plot.py <csv_file> [0,1,2,3]")
    exit(-1)

if len(sys.argv) > 2:
    bp0 = sys.argv[2].find('0') != -1
    bp1 = sys.argv[2].find('1') != -1
    bp2 = sys.argv[2].find('2') != -1
    bp3 = sys.argv[2].find('3') != -1
else:
    bp0 = True
    bp1 = True
    bp2 = True
    bp3 = True

headers = ["Phase 0 Allocations", "Phase 0 Deallocations", "Phase 1 Allocations", "Phase 1 Deallocations",
        "Phase 2 Allocations", "Phase 2 Deallocations", "Phase 3 Allocations", "Phase 3 Deallocations"]

df = pd.read_csv(sys.argv[1], delimiter=",", names=headers)
p0a = [float(value) for value in df["Phase 0 Allocations"].values[1::]]
p1a = [float(value) for value in df["Phase 1 Allocations"].values[1::]]
p2a = [float(value) for value in df["Phase 2 Allocations"].values[1::]]
p3a = [float(value) for value in df["Phase 3 Allocations"].values[1::]]

axis = np.arange(1, len(p0a)+1)

style.use('ggplot')

if bp0:
    plt.plot(axis, p0a, label='Ph0')
if bp1:
    plt.plot(axis, p1a, label='Ph1')
if bp2:
    plt.plot(axis, p2a, label='Ph2')
if bp3:
    plt.plot(axis, p3a, label='Ph3')
plt.xlabel("Execution")
plt.ylabel("Allocations")
plt.title("Allocations in phases")
plt.xticks(axis, axis)
plt.legend()
plt.show()



#!/usr/bin/env python3

# Copyright 2020 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
    fastdds CLI tool.

    This CLI tool provide a set commands and sub-commands to perform, Fast-DDS
    related, maintenance / configuration tasks.

    usage: fastdds <command> [<command-args>]

        Commands:

            discovery     Server-Client discovery auxiliary generator

            shm           Shared-memory commands

        fastdds <command> [-h] shows command usage


    positional arguments:
    command     Command to run

    optional arguments:
    -h, --help  show this help message and exit

"""

import argparse
import sys

from discovery.parser import Parser as DiscoveryParser

from shm.parser import Parser as ShmParser


class FastDDSParser:
    """FastDDS tool parser."""

    __required_python_version = (3, 5)

    __help_message = """fastdds <command> [<command-args>]\n\n
    Commands:\n\n
    \tdiscovery     Server-Client discovery auxiliary generator\n
    \tshm           Shared-memory commands\n
    fastdds <command> [-h] shows command usage
    """

    def __init__(self):
        """Parse sys.argv[1:2].

        Parses the <command> and dispatches to the appropriate handler.
        Shows usage if no command is specified.
        """
        self.__check_python_version()

        parser = argparse.ArgumentParser(
            usage=self.__help_message,
            add_help=True
        )

        parser.add_argument('command',
                            nargs='?',
                            help='Command to run')

        args = parser.parse_args(sys.argv[1:2])

        if args.command is not None:
            if not hasattr(self, args.command):
                print('Invalid command')
            else:
                getattr(self, args.command)()
        else:
            parser.print_help()

    def __check_python_version(self):
        """Assert python version is valid."""
        req_v = self.__required_python_version
        v = sys.version_info
        if not (
                    ((v[0] == req_v[0]) and (v[1] >= req_v[1])) or
                    (v[0] > req_v[0])
               ):
            print('fastdds: Invalid Python version. {}.{}.x or greater'
                  ' is required'.format(req_v[0], req_v[1]))
            sys.exit(1)

    def shm(self):
        """Shared-memory command handler."""
        ShmParser(sys.argv[2:])

    def discovery(self):
        """Discovery server command handler."""
        DiscoveryParser(sys.argv[2:])


if __name__ == '__main__':

    FastDDSParser()



"""Execute a DDS communictaion test following a json definition file."""
# Copyright 2021 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import glob
import json
import logging
import os
import subprocess
import sys

script_dir = os.path.dirname(os.path.realpath(__file__))
seed = str(os.getpid())


def read_json(file_name):
    """Read json file with test definition."""
    structure_dic = {}
    with open(file_name) as json_file:
        structure_dic = json.load(json_file)
    return structure_dic


def test_definition(file_name):
    """Return a list with each test process defined in dictionary."""
    return read_json(file_name)['participants']


def define_args(tests_definition):
    """Use list dictionary to get commands args for each test process."""
    sub_args = []
    pub_args = []
    pubsub_args = []

    for test in tests_definition:

        if 'kind' not in test.keys():
            print('ARGUMENT ERROR : '
                  'Test definition requites <kind> field for each participant')
            continue

        # All processes has seed argument
        test_arguments = ['--seed', seed]

        possible_arguments = ['samples',
                              'wait',
                              'magic',
                              'publishers']

        for argument in possible_arguments:
            if argument in test.keys():
                test_arguments.append('--' + argument)
                test_arguments.append(test[argument])

        # Add arguments without value
        possible_flags = ['exit_on_lost_liveliness',
                          'zero_copy',
                          'fixed_type',
                          'notexit']

        for flag in possible_flags:
            if flag in test.keys():
                test_arguments.append('--' + flag)

        xmlfile_arg = 'xmlfile'
        if xmlfile_arg in test.keys():
            xml_path = os.path.join(script_dir, test[xmlfile_arg])
            test_arguments.append('--' + xmlfile_arg)
            test_arguments.append(xml_path)

        if test['kind'] == 'publisher':
            pub_args.append(test_arguments)

        elif test['kind'] == 'subscriber':
            sub_args.append(test_arguments)

        elif test['kind'] == 'pubsub':
            pubsub_args.append(test_arguments)

        else:
            print('ARGUMENT ERROR : '
                  '<kind> field can be publisher/subscriber/pubsub')

    return pub_args, sub_args, pubsub_args


def define_commands(pub_args, sub_args, pubsub_args):
    """Create commands for each test addind executable to args."""
    # Publisher executable
    publisher_command = os.environ.get(
        'DDS_SIMPLE_COMMUNICATION_PUBLISHER_BIN')
    if not publisher_command:
        publisher_files = glob.glob(
            os.path.join(
                script_dir,
                '**/DDSCommunicationPublisher*'),
            recursive=True)
        pf = iter(publisher_files)
        publisher_command = next(pf, None)
        while publisher_command and \
                (not os.path.isfile(publisher_command)
                 or not os.access(publisher_command,
                 os.X_OK)):
            publisher_command = next(pf, None)
    assert publisher_command

    # Subscriber executable
    subscriber_command = os.environ.get(
        'DDS_SIMPLE_COMMUNICATION_SUBSCRIBER_BIN')
    if not subscriber_command:
        subscriber_files = glob.glob(
            os.path.join(
                script_dir,
                '**/DDSCommunicationSubscriber*'),
            recursive=True)
        pf = iter(subscriber_files)
        subscriber_command = next(pf, None)
        while subscriber_command and \
            (not os.path.isfile(subscriber_command)
             or not os.access(subscriber_command,
             os.X_OK)):
            subscriber_command = next(pf, None)
    assert subscriber_command

    # Pub Sub executable
    pubsub_command = os.environ.get('DDS_SIMPLE_COMMUNICATION_PUBSUB_BIN')
    if not pubsub_command:
        pubsub_files = glob.glob(
            os.path.join(
                script_dir,
                '**/DDSCommunicationPubSub*'),
            recursive=True)
        pf = iter(pubsub_files)
        pubsub_command = next(pf, None)
        while pubsub_command and \
            (not os.path.isfile(pubsub_command)
             or not os.access(pubsub_command,
             os.X_OK)):
            pubsub_command = next(pf, None)
    assert pubsub_command

    # Add executable to each command
    return (
        [[publisher_command] + args for args in pub_args],
        [[subscriber_command] + args for args in sub_args],
        [[pubsub_command] + args for args in pubsub_args]
    )


def execute_commands(pub_commands, sub_commands, pubsub_commands, logger):
    """Get test definitions in command lists and execute each process."""
    pubs_proc = []
    subs_proc = []
    pubsubs_proc = []

    for subscriber_command in sub_commands:
        logger.info(f'Executing subcriber: {subscriber_command}')
        proc = subprocess.Popen(subscriber_command)
        subs_proc.append(proc)

    for pubsub_command in pubsub_commands:
        logger.info(f'Executing pubsub: {pubsub_command}')
        proc = subprocess.Popen(pubsub_command)
        pubsubs_proc.append(proc)

    for publisher_command in pub_commands:
        logger.info(f'Executing publisher: {publisher_command}')
        proc = subprocess.Popen(publisher_command)
        pubs_proc.append(proc)

    ret_value = 0

    for proc in subs_proc:
        proc.communicate()
        ret_value = ret_value + proc.returncode

    for proc in pubsubs_proc:
        proc.communicate()
        ret_value = ret_value + proc.returncode

    for proc in pubs_proc:
        proc.kill()

    return ret_value


if __name__ == '__main__':

    logger = logging.getLogger('DDS COMMUNICATION TEST')
    logger.setLevel(logging.INFO)

    logger.error("TEST RUNNING")

    args = sys.argv[1:]

    if len(args) != 1:
        logger.error('ARGUMENTS ERROR : 1 argument required: '
                     'path to .json file with test definition')
        sys.exit(1)

    test_definitions = test_definition(args[0])

    logger.error(test_definition)

    pub_args, sub_args, pubsub_args = define_args(test_definitions)

    logger.error(pub_args)
    logger.error(sub_args)
    logger.error(pubsub_args)

    pub_commands, sub_commands, pubsub_commands = \
        define_commands(pub_args, sub_args, pubsub_args)

    logger.error(pub_commands)
    logger.error(sub_commands)
    logger.error(pubsub_commands)

    test_value = execute_commands(
        pub_commands,
        sub_commands,
        pubsub_commands,
        logger)

    logger.error(test_value)

    if test_value != 0:
        sys.exit(1)
    else:
        sys.exit(0)



import sys
import msparser
import csv

stack = []
heap = []
msparser_data = msparser.parse_file(sys.argv[1])
for snapshot in msparser_data['snapshots']:
    if snapshot['mem_heap'] != 0:
        stack.append(snapshot['mem_stack'])
        heap.append(snapshot['mem_heap'])

with open(sys.argv[2], 'w+') as csv_file:
    csv_writer = csv.writer(csv_file, delimiter=',', quoting=csv.QUOTE_ALL)
    csv_writer.writerow(['stack', 'heap'])
    csv_writer.writerow([max(stack), max(heap)])


# Copyright 2016 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import shlex, subprocess, time, os, socket, sys, threading, glob, csv

if os.environ.get("PROFILING_BINS"):
    binaries = os.environ.get("PROFILING_BINS").split(';')

valgrind = os.environ.get("VALGRIND_BIN")
certs_path = os.environ.get("CERTS_PATH")
test_time = "10"

if not valgrind:
    valgrind = "valgrind"

def cycles_result(filename, thread_logs):
    thread_numbers = []
    thread_instruction_counts = []
    for thread_log in thread_logs:
        with open (thread_log, 'r') as f:
            lines = f.read().splitlines()
            thread_numbers.append(lines[2].split(' ')[-1])
            thread_instruction_counts.append(int(lines[4].split(' ')[-1]))
        os.remove(thread_log)

    total_instructions = sum(thread_instruction_counts)
    with open('./output/results_' + filename + '.csv','w') as results:
        writer = csv.writer(results)
        writer.writerow(['Context','Instructions'])
        writer.writerow(['Total', total_instructions])
        for number, count in zip(thread_numbers, thread_instruction_counts):
            writer.writerow(['Thread ' + number, str(count)])

def start_test(command, pubsub, time):
    os.system("mkdir -p output")

    valgrind_command_rel = [valgrind, "--tool=exp-bbv","--instr-count-only=yes", "--bb-out-file=./output/bbLog_" + pubsub + "_rel.out"]
    valgrind_command_be = [valgrind, "--tool=exp-bbv","--instr-count-only=yes", "--bb-out-file=./output/bbLog_" + pubsub + "_be.out"]

    options = ["--time=" + time]

    if certs_path:
        options.extend(["--security=true", "--certs=" + certs_path])

    # Best effort
    proc = subprocess.Popen(valgrind_command_be +
            [command, pubsub] +
            options)

    proc.communicate()

    # Reliable
    proc = subprocess.Popen(valgrind_command_rel +
            [command, pubsub, "-r", "reliable"] +
            options)

    proc.communicate()

    thread_logs_pub_rel = glob.glob('./output/bbLog_publisher_rel.out')
    thread_logs_sub_rel = glob.glob('./output/bbLog_subscriber_rel.out')
    thread_logs_pub_be = glob.glob('./output/bbLog_publisher_be.out')
    thread_logs_sub_be = glob.glob('./output/bbLog_subscriber_be.out')

    cycles_result("pub_rel", thread_logs_pub_rel)
    cycles_result("sub_rel", thread_logs_sub_rel)
    cycles_result("pub_be", thread_logs_pub_be)
    cycles_result("sub_be", thread_logs_sub_be)

if len(sys.argv) >= 4:
    test_time = sys.argv[3]

if len(sys.argv) == 3:
    binaries = [sys.argv[2]]

for command in binaries:
    if len(sys.argv) == 2:
        pubsub = sys.argv[1]
        start_test(command, pubsub, test_time)
    else:
        tpub = threading.Thread(target=start_test, args=(command, "publisher", test_time))
        tpub.start()
        tsub = threading.Thread(target=start_test, args=(command, "subscriber", test_time))
        tsub.start()

quit()



# Copyright 2016 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import os
import subprocess

if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument(
        '-x',
        '--xml_file',
        help='A Fast-RTPS XML configuration file',
        required=False
    )
    parser.add_argument(
        '-f',
        '--demands_file',
        help='Filename of the demands configuration file',
        required=False,
        default=None
    )
    parser.add_argument(
        '-n',
        '--number_of_samples',
        help='The number of measurements to take for each payload',
        required=False,
        default='10000'
    )
    parser.add_argument(
        '-s',
        '--security',
        action='store_true',
        help='Enables security (Defaults: disable)',
        required=False
    )
    parser.add_argument(
        '-i',
        '--interprocess',
        action='store_true',
        help='Publisher and subscribers in separate processes. Defaults:False',
        required=False
    )
    parser.add_argument(
        '-d',
        '--data_sharing',
        choices=['on', 'off'],
        help='Explicitly enable/disable data sharing. (Defaults: Fast-DDS default settings)',
        required=False
    )
    parser.add_argument(
        '-l',
        '--data_loans',
        action='store_true',
        help='Enable the use of the loan sample API (Defaults: disable)',
        required=False
    )
    parser.add_argument(
        '-r',
        '--reliability',
        action='store_true',
        help='Run with RELIABLE reliability (Defaults: disable)',
        required=False
    )
    parser.add_argument(
        '--shared_memory',
        choices=['on', 'off'],
        help='Explicitly enable/disable shared memory transport. (Defaults: Fast-DDS default settings)',
        required=False
        )

    # Parse arguments
    args = parser.parse_args()
    xml_file = args.xml_file
    security = args.security
    interprocess = args.interprocess

    if security and not interprocess:
        print('Intra-process delivery NOT supported with security')
        exit(1)  # Exit with error

    # Check that samples is positive
    if str.isdigit(args.number_of_samples) and int(args.number_of_samples) > 0:
        samples = str(args.number_of_samples)
    else:
        print(
            '"number_of_samples" must be a positive integer, NOT {}'.format(
                args.number_of_samples
            )
        )
        exit(1)  # Exit with error

    # Demands files options
    demands_options = []
    if args.demands_file:
        if not os.path.isfile(args.demands_file):
            print('Demands file "{}" is NOT a file'.format(args.demands_file))
            exit(1)  # Exit with error
        else:
            demands_options = [
                '--file',
                args.demands_file,
            ]

    # XML options
    filename_options = 'default'
    xml_options = []
    if xml_file:
        if not os.path.isfile(xml_file):
            print('XML file "{}" is NOT a file'.format(xml_file))
            exit(1)  # Exit with error
        else:
            xml_options = ['--xml', xml_file]
            # Get QoS from XML
            filename_options = xml_file.split('/')[-1].split('\\')[-1]
            filename_options = filename_options.split('.')[-2].split('_')[1:]
            filename_options = '_'.join(filename_options)

    # Data sharing and loans options
    # modify output file names
    if args.data_sharing and 'on' == args.data_sharing and args.data_loans:
        filename_options += '_data_loans_and_sharing'
    elif args.data_sharing and 'on' == args.data_sharing:
        filename_options += '_data_sharing'
    elif args.data_loans:
        filename_options += '_data_loans'

    # add flags to the command line
    data_options = []

    if args.data_sharing:
        if 'on' == args.data_sharing:
            data_options += ['--data_sharing=on']
        else:
            data_options += ['--data_sharing=off']

    if args.data_loans:
        data_options += ['--data_loans']

    reliability_options = []
    if args.reliability:
        reliability_options = ['--reliability=reliable']
    else:
        reliability_options = ['--reliability=besteffort']

    if args.shared_memory:
        if 'on' == args.shared_memory:
            data_options += ['--shared_memory=on']
        else:
            data_options += ['--shared_memory=off']

    # Environment variables
    executable = os.environ.get('LATENCY_TEST_BIN')
    certs_path = os.environ.get('CERTS_PATH')

    # Check that executable exists
    if executable:
        if not os.path.isfile(executable):
            print('LATENCY_TEST_BIN does NOT specify a file')
            exit(1)  # Exit with error
    else:
        print('LATENCY_TEST_BIN is NOT set')
        exit(1)  # Exit with error

    # Security
    security_options = []
    if security is True:
        if certs_path:
            if os.path.isdir(certs_path):
                security_options = ['--security=true', '--certs=' + certs_path]
            else:
                print('CERTS_PATH does NOT specify a directory')
                exit(1)  # Exit with error
        else:
            print('Cannot find CERTS_PATH environment variable')
            exit(1)  # Exit with error

    # Domain
    domain = str(os.getpid() % 230)
    domain_options = ['--domain', domain]

    if interprocess is True:
        # Base of test command for publisher agent
        pub_command = [
            executable,
            'publisher',
            '--samples',
            samples,
            '--export_raw_data',
        ]
        # Base of test command for subscriber agent
        sub_command = [
            executable,
            'subscriber',
        ]

        # Manage security
        if security is True:
            pub_command.append(
                './measurements_interprocess_{}_security.csv'.format(
                    filename_options
                )
            )
            pub_command += security_options
            sub_command += security_options
        else:
            pub_command.append(
                './measurements_interprocess_{}.csv'.format(
                    filename_options
                )
            )

        pub_command += domain_options
        pub_command += xml_options
        pub_command += demands_options
        pub_command += data_options
        pub_command += reliability_options

        sub_command += domain_options
        sub_command += xml_options
        sub_command += demands_options
        sub_command += data_options
        sub_command += reliability_options

        print('Publisher command: {}'.format(
            ' '.join(element for element in pub_command)),
            flush=True
        )
        print('Subscriber command: {}'.format(
            ' '.join(element for element in sub_command)),
            flush=True
        )

        # Spawn processes
        publisher = subprocess.Popen(pub_command)
        subscriber = subprocess.Popen(sub_command)
        # Wait until finish
        subscriber.communicate()
        publisher.communicate()

        if subscriber.returncode != 0:
            exit(subscriber.returncode)
        elif publisher.returncode != 0:
            exit(publisher.returncode)
    else:
        # Base of test command to execute
        command = [
            executable,
            'both',
            '--samples',
            samples,
            '--export_raw_data',
        ]

        # Manage security
        if security is True:
            command.append(
                './measurements_intraprocess_{}_security.csv'.format(
                    filename_options
                )
            )
            command += security_options
        else:
            command.append(
                './measurements_intraprocess_{}.csv'.format(
                    filename_options
                )
            )

        command += domain_options
        command += xml_options
        command += demands_options
        command += data_options
        command += reliability_options

        print('Executable command: {}'.format(
            ' '.join(element for element in command)),
            flush=True
        )

        # Spawn process
        both = subprocess.Popen(command)
        # Wait until finish
        both.communicate()
        exit(both.returncode)
    exit(0)



# Copyright 2020 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Script to test the secure communication with encrypted RTPS messages."""
import argparse
import os
import subprocess
import sys


class ParseOptions():
    """Parse arguments."""

    def __init__(self):
        """Object constructor."""
        self.args = self.__parse_args()

    def __parse_args(self):
        """
        Parse the input arguments.

        :return: A dictionary containing the arguments parsed.
        """
        parser = argparse.ArgumentParser(
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
            add_help=True,
            description=(
                'Script to test the secure communication with encrypted RTPS'
                'messages.'),
        )
        parser.add_argument(
            '-p',
            '--pub',
            type=str,
            required=True,
            help='Path to the Publisher executable.'
        )
        parser.add_argument(
            '-s',
            '--sub',
            type=str,
            required=True,
            help='Path to the Subscriber executable.'
        )
        parser.add_argument(
            '-P',
            '--xml-pub',
            type=str,
            help='Path to the publisher xml configuration file.'
        )
        parser.add_argument(
            '-S',
            '--xml-sub',
            type=str,
            help='Path to the subscriber xml configuration file.'
        )
        parser.add_argument(
            '-x',
            '--xml',
            type=str,
            help=(
                'Path to the xml configuration file containing publisher and'
                'subscriber profiles.')
        )
        parser.add_argument(
            '-w',
            '--wait',
            type=int,
            help='Time for the publisher to wait for discovery.'
        )
        parser.add_argument(
            '-a',
            '--samples',
            type=int,
            help='Number of samples sent by the publisher.'
        )
        parser.add_argument(
            '-n',
            '--n-subs',
            type=int,
            help='Number of subscribers to launch.'
        )

        return parser.parse_args()


def run(args):
    """
    Run the publisher and both susbcribers.

    :param args: The input paramenters.

    :return: The return code resulting from the publisher and subscribers
        execution. It is the number of failed processes.
    """
    pub_command = []
    sub_command = []

    script_dir = os.path.dirname(os.path.realpath(__file__))

    if not os.path.isfile(args.pub):
        print(f'Publisher executable file does not exists: {args.pub}')
        sys.exit(1)

    if not os.access(args.pub, os.X_OK):
        print(
            'Publisher executable does not have execution permissions:'
            f'{args.pub}')

    pub_command.append(args.pub)

    if not os.path.isfile(args.sub):
        print(f'Subscriber executable file does not exists: {args.sub}')
        sys.exit(1)

    if not os.access(args.sub, os.X_OK):
        print(
            'Subscriber executable does not have execution permissions:'
            f'{args.sub}')
        sys.exit(1)

    sub_command.append(args.sub)

    if args.xml:
        xml_file_pub = os.path.join(script_dir, args.xml)
        xml_file_sub = os.path.join(script_dir, args.xml)
    elif args.xml_pub and args.xml_sub:
        if args.xml_pub:
            xml_file_pub = os.path.join(script_dir, args.xml_pub)
        if args.xml_sub:
            xml_file_sub = os.path.join(script_dir, args.xml_sub)
    else:
        print('Not provided xml configuration files.')
        sys.exit(1)

    pub_command.extend(['--xmlfile', xml_file_pub])
    sub_command.extend(['--xmlfile', xml_file_sub])

    pub_command.extend(['--seed', str(os.getpid())])
    sub_command.extend(['--seed', str(os.getpid())])

    if args.wait:
        pub_command.extend(['--wait', str(args.wait)])

    if args.samples:
        pub_command.extend(['--samples', str(args.samples)])
        sub_command.extend(['--samples', str(args.samples)])

    sub_processes = []
    for i in range(args.n_subs):
        sub_processes.append(subprocess.Popen(sub_command))
        print(
            f'Running Subscriber {i+1} - commmand:  ',
            ' '.join(map(str, sub_command)))

    pub_proc = subprocess.Popen(pub_command)
    print(
        'Running Publisher - commmand:  ',
        ' '.join(map(str, pub_command)))

    failed_subs = 0
    for sub, sub_proc in enumerate(sub_processes):
        sub_proc.communicate()
        if sub_proc.returncode != 0:
            print(f'Subscriber {sub+1} failed while running.')
            failed_subs += 1

    pub_proc.kill()

    sys.exit(failed_subs)


if __name__ == '__main__':

    # Parse arguments
    args = ParseOptions()

    run(args.args)



# Copyright 2016 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys, os, subprocess, glob, time

script_dir = os.path.dirname(os.path.realpath(__file__))

publisher_command = os.environ.get("SIMPLE_COMMUNICATION_PUBLISHER_BIN")
if not publisher_command:
    publisher_files = glob.glob(os.path.join(script_dir, "**/SimpleCommunicationPublisher*"), recursive=True)
    pf = iter(publisher_files)
    publisher_command = next(pf, None)
    while publisher_command and (not os.path.isfile(publisher_command) or not os.access(publisher_command,
        os.X_OK)):
        publisher_command = next(pf, None)
assert publisher_command
subscriber_command = os.environ.get("SIMPLE_COMMUNICATION_SUBSCRIBER_BIN")
if not subscriber_command:
    subscriber_files = glob.glob(os.path.join(script_dir, "**/SimpleCommunicationSubscriber*"), recursive=True)
    pf = iter(subscriber_files)
    subscriber_command = next(pf, None)
    while subscriber_command and (not os.path.isfile(subscriber_command) or not os.access(subscriber_command,
        os.X_OK)):
        subscriber_command = next(pf, None)
assert subscriber_command

extra_pub_arg = os.environ.get("EXTRA_PUB_ARG")
if extra_pub_arg:
    extra_pub_args = extra_pub_arg.split()
else:
    extra_pub_args = []

real_xml_file_pub = None
real_xml_file_sub = None
xml_file = os.environ.get("XML_FILE")
if xml_file:
    real_xml_file_pub = os.path.join(script_dir, xml_file)
    real_xml_file_sub = os.path.join(script_dir, xml_file)
else:
    xml_file_pub = os.environ.get("XML_FILE_PUB")
    if xml_file_pub:
        real_xml_file_pub = os.path.join(script_dir, xml_file_pub)
    xml_file_sub = os.environ.get("XML_FILE_SUB")
    if xml_file_sub:
        real_xml_file_sub = os.path.join(script_dir, xml_file_sub)

subscriber1_proc = subprocess.Popen([subscriber_command, "--seed", str(os.getpid()), "--publishers", "2", "--samples", "10"]
        + (["--xmlfile", real_xml_file_sub] if real_xml_file_sub else []))
publisher1_proc = subprocess.Popen([publisher_command, "--seed", str(os.getpid()), "--samples", "10"]
        + (["--xmlfile", real_xml_file_pub] if real_xml_file_pub else [])
        + extra_pub_args)
time.sleep(1)
publisher2_proc = subprocess.Popen([publisher_command, "--seed", str(os.getpid()), "--samples", "10"]
        + (["--xmlfile", real_xml_file_pub] if real_xml_file_pub else [])
        + extra_pub_args)
subscriber2_proc = subprocess.Popen([subscriber_command, "--seed", str(os.getpid()), "--publishers", "2", "--samples", "10"]
        + (["--xmlfile", real_xml_file_sub] if real_xml_file_sub else []))

subscriber1_proc.communicate()
retvalue = subscriber1_proc.returncode
subscriber2_proc.communicate()
if retvalue == 0:
    retvalue = subscriber2_proc.returncode

publisher1_proc.kill()
publisher2_proc.kill()

sys.exit(retvalue)



# Copyright 2016 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import subprocess
import os


def assert_positive_int(str_number, str_name):
    """
    Check whether a string represents a positive integer.

    :param str_number: The string representation.
    :param str_name: The name to print on error.
    :return: The string representation if it was a positive interger.
        Exits with error code 12 otherwise.
    """
    # Check that samples is positive
    if str.isdigit(str_number) and int(str_number) > 0:
        return str(str_number)
    else:
        print(
            '"{}" must be a positive integer, NOT {}'.format(
                str_name,
                str_number
            )
        )
        exit(1)  # Exit with error


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument(
        '-x',
        '--xml_file',
        help='A Fast-RTPS XML configuration file',
        required=False
    )
    parser.add_argument(
        '-n',
        '--number_of_samples',
        help='The number of measurements to take for each payload',
        required=False,
        default='10000'
    )
    parser.add_argument(
        '-H',
        '--height',
        help='Height of the video',
        required=False,
        default='720'
    )
    parser.add_argument(
        '-w',
        '--width',
        help='Width of the video',
        required=False,
        default='1024'
    )
    parser.add_argument(
        '-r',
        '--frame_rate',
        help='Frame rate of the video [Hz]',
        required=False,
        default='30'
    )
    parser.add_argument(
        '-t',
        '--test_duration',
        help='The duration of the test [s]',
        required=False,
        default=2
    )
    parser.add_argument(
        '-s',
        '--security',
        action='store_true',
        help='Enables security (Defaults: disable)',
        required=False
    )
    # Parse arguments
    args = parser.parse_args()
    xml_file = args.xml_file
    security = args.security

    # XML options
    xml_options = []
    if xml_file:
        if not os.path.isfile(xml_file):
            print('XML file "{}" is NOT a file'.format(xml_file))
            exit(1)  # Exit with error
        else:
            xml_options = ['--xml', xml_file]

    samples = assert_positive_int(str(args.number_of_samples), "number_of_samples")
    height = assert_positive_int(str(args.height), "height")
    width = assert_positive_int(str(args.width), "width")
    frame_rate = assert_positive_int(str(args.frame_rate), "frame_rate")
    test_duration = assert_positive_int(str(args.test_duration), "test_duration")

    # Environment variables
    executable = os.environ.get('VIDEO_TEST_BIN')
    certs_path = os.environ.get('CERTS_PATH')

    # Check that executable exists
    if executable:
        if not os.path.isfile(executable):
            print('VIDEO_TEST_BIN does NOT specify a file')
            exit(1)  # Exit with error
    else:
        print('VIDEO_TEST_BIN is NOT set')
        exit(1)  # Exit with error

    # Security
    security_options = []
    if security is True:
        if certs_path:
            if os.path.isdir(certs_path):
                security_options = ['--security=true', '--certs=' + certs_path]
            else:
                print('CERTS_PATH does NOT specify a directory')
                exit(1)  # Exit with error
        else:
            print('Cannot find CERTS_PATH environment variable')
            exit(1)  # Exit with error

    # Domain
    domain = str(os.getpid() % 230)
    domain_options = ['--domain', domain]

    # Base of test command for publisher agent
    pub_command = [
        executable,
        'publisher',
        '--samples',
        samples,
        '--testtime',
        test_duration,
        '--width',
        width,
        '--height',
        height,
        '--rate',
        frame_rate
    ]
    # Base of test command for subscriber agent
    sub_command = [
        executable,
        'subscriber',
    ]

    # Manage security
    if security is True:
        pub_command += security_options
        sub_command += security_options

    pub_command += domain_options
    pub_command += xml_options
    sub_command += domain_options
    sub_command += xml_options


    print('Publisher command: {}'.format(
            ' '.join(element for element in pub_command)),
            flush=True
    )
    print('Subscriber command: {}'.format(
            ' '.join(element for element in sub_command)),
            flush=True
    )

    # Spawn processes
    publisher = subprocess.Popen(pub_command)
    subscriber = subprocess.Popen(sub_command)
    # Wait until finish
    subscriber.communicate()
    publisher.communicate()

    if subscriber.returncode != 0:
            exit(subscriber.returncode)
    elif publisher.returncode != 0:
            exit(publisher.returncode)



# Copyright 2020 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
    fastdds discovery verb parser.

    The parser just forward the sub-commands to the fast-discovery-server
    tool application.

"""

import os
import re
import subprocess
import sys
from pathlib import Path


class Parser:
    """Discovery server tool parser."""

    __help_message = 'fastdds discovery <discovery-args>\n\n'

    def __init__(self, argv):
        """
        Parse the sub-command and dispatch to the appropriate handler.

        Shows usage if no sub-command is specified.

        """
        tool_path = str(self.__find_tool_path().resolve())

        try:
            result = subprocess.run(
                [tool_path],
                stdout=subprocess.PIPE,
                universal_newlines=True
            )

            if result.returncode != 0:
                sys.exit(result.returncode)

            if (
                    len(argv) == 0 or
                    (len(argv) == 1 and argv[0] == '-h') or
                    (len(argv) == 1 and argv[0] == '--help')
               ):
                print(self.__edit_tool_help(result.stdout))
            else:
                # Call the tool
                result = subprocess.run([tool_path] + argv)
                if result.returncode != 0:
                    sys.exit(result.returncode)

        except KeyboardInterrupt as e:
            # it lets the subprocess to handle the exception
            pass

        except BaseException as e:
            self.__help_message += str(e)
            self.__help_message += '\n fast-discovery-server tool not found!'
            print(self.__help_message)
            sys.exit(1)

    def __find_tool_path(self):
        """
        Calculate the path to the fast-discovery-server tool.

        returns str:
            Full path to the executable

        """
        tool_path = Path(os.path.dirname(os.path.realpath(__file__)))
        # We asume the installion path is relative to our installation path
        tool_path = tool_path / '../../../bin'
        if os.name == 'posix':
            ret = tool_path / 'fast-discovery-server'
            if not os.path.exists(ret):
                print('fast-discovery-server tool not installed')
                sys.exit(1)
        elif os.name == 'nt':
            ret = tool_path / 'fast-discovery-server.exe'
            if not os.path.exists(ret):
                ret = tool_path / 'fast-discovery-server.bat'
                if not os.path.exists(ret):
                    print('fast-discovery-server tool not installed')
                    sys.exit(1)
        else:
            print(f'{os.name} not supported')
            sys.exit(1)

        return ret

    def __edit_tool_help(self, usage_text):
        """Find and replace the tool-name by fastdds discovery."""
        m = re.search('Usage: ([a-zA-Z0-9-\\.]*)\\s', usage_text)
        if m:
            tool_name = m.group(1)
            return re.sub(tool_name, 'fastdds discovery', usage_text)

        return usage_text



# Copyright 2016 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import shlex, subprocess, time, os, socket, sys, threading

if os.environ.get("PROFILING_BINS"):
    binaries = os.environ.get("PROFILING_BINS").split(';')

valgrind = os.environ.get("VALGRIND_BIN")
certs_path = os.environ.get("CERTS_PATH")
test_time = "10"

if not valgrind:
    valgrind = "valgrind"

def start_test(command, pubsub, time, transport):
    os.system("mkdir -p output")

    valgrind_command_rel = [valgrind, "--tool=massif", "--stacks=yes", "--detailed-freq=1", "--max-snapshots=1000", "--massif-out-file=./output/consumption_" + pubsub + "_" + transport + "_rel.out"]
    valgrind_command_be = [valgrind, "--tool=massif", "--stacks=yes", "--detailed-freq=1", "--max-snapshots=1000", "--massif-out-file=./output/consumption_" + pubsub + "_" + transport + "_be.out"]

    options = ["--time=" + time]

    if certs_path:
        options.extend(["--security=true", "--certs=" + certs_path])

    # Best effort
    print(valgrind_command_be +
            [command, pubsub] +
            options)
    proc = subprocess.Popen(valgrind_command_be +
            [command, pubsub] +
            options)

    proc.communicate()

    py_command = "python3 ./memory_analysis.py ./output/consumption_" + pubsub + "_" + transport + "_be.out ./output/MemoryTest_" + pubsub + "_" + transport + "_be.csv"
    p = subprocess.Popen(py_command, shell=True)

    # Reliable
    proc = subprocess.Popen(valgrind_command_rel +
            [command, pubsub, "-r", "reliable"] +
            options)

    proc.communicate()

    py_command = "python3 ./memory_analysis.py ./output/consumption_" + pubsub + "_" + transport + "_rel.out ./output/MemoryTest_" + pubsub + "_" + transport + "_rel.csv"
    # print("Command: " + py_command)
    p = subprocess.Popen(py_command, shell=True)

transport = ""

if len(sys.argv) >= 5:
    transport = sys.argv[4]

if len(sys.argv) >= 4:
    test_time = sys.argv[3]

if len(sys.argv) >= 3:
    binaries = [sys.argv[2]]

for command in binaries:
    if len(sys.argv) >= 2:
        pubsub = sys.argv[1]
        start_test(command, pubsub, test_time, transport)
    else:
        tpub = threading.Thread(target=start_test, args=(command, "publisher", test_time, transport))
        tpub.start()
        tsub = threading.Thread(target=start_test, args=(command, "subscriber", test_time, transport))
        tsub.start()

quit()



# Copyright 2020 Proyectos y Sistemas de Mantenimiento SL (eProsima).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
    Sub-Command Clean implementation.

    This sub-command finds and remove unused shared-memory files.

"""

import os
import platform
import re
from pathlib import Path

if os.name == 'posix':
    import fcntl
elif os.name == 'nt':
    import win32con
    import win32file
    import pywintypes
    import msvcrt


class Clean:
    """This command searches and deletes zombie SHM ports / segments."""

    def run(self):
        """Execute the clean."""
        self.__ports_in_use = 0
        self.__segments_in_use = 0

        zombie_segments = self.__clean_zombie_segments()
        zombie_ports = self.__clean_zombie_ports()

        print('shm.clean:')
        print(self.__ports_in_use, 'ports in use')
        print(self.__segments_in_use, 'segments in use')
        # each port has 3 files
        print(int(len(zombie_ports) / 3), 'zombie ports cleaned')
        # each segment has 2 files
        print(int(len(zombie_segments) / 2), 'zombie segments cleaned')

    def __shm_dir(self):
        """
        Calculate the shm directory.

        returns Path:
            The path to the platform specific the SHM directory

        """
        # Windows
        if os.name == 'nt':
            shm_path = Path('c:\\programdata\\eprosima\\'
                            'fastrtps_interprocess\\').resolve()
        elif os.name == 'posix':
            # MAC
            if platform.mac_ver()[0] != '':
                shm_path = Path('/private/tmp/boost_interprocess/').resolve()
            # Linux
            else:
                shm_path = Path('/dev/shm/').resolve()
        else:
            raise RuntimeError(f'{os.name} not supported')

        return shm_path

    def __list_dir(self):
        """Return a list of files in the default SHM dir."""
        try:
            return os.listdir(self.__shm_dir())
        except BaseException:
            return []

    def __clean_zombie_segments(self):
        """
        Find & delete zombie segments in the default SHM dir.

        returns list(str):
            The deleted file names

        """
        segment_lock_re = re.compile('^fastrtps_(\\d|[a-z]){16}_el|_sl')

        # Each segment has an "_el" lock file that is locked if the segment
        # is open and the owner process is alive
        segment_locks = [
            str(self.__shm_dir() / file_name) for file_name in
            self.__list_dir() if segment_lock_re.match(file_name)]
        zombie_files = []

        # Check is_file_locked for each lock file
        for f in segment_locks:
            if not self.__is_file_locked(f):
                # Not locked so delete lock file & segment file
                segment_file = f
                lock_file = f[:-3]
                self.__remove_file(segment_file)
                self.__remove_file(lock_file)
                f = [segment_file, lock_file]
                zombie_files += f
            else:
                self.__segments_in_use += 1

        return zombie_files

    def __clean_zombie_ports(self):
        """
        Find & delete zombie ports in the default SHM dir.

        returns list(str):
            the deleted file names

        """
        port_lock_re = re.compile('^fastrtps_port\\d{,5}_el|_sl')
        # Each port has an "_el | _sl" lock file that is locked if the port
        # is open and the owner process is alive
        port_locks = [
            file_name for file_name in self.__list_dir() if port_lock_re.match(
                file_name)]
        zombie_files = []

        # Check is_file_locked for each lock file
        for file in port_locks:
            if not self.__is_file_locked(self.__shm_dir() / file):
                # Not locked so delete lock file, segment file and
                # port_mutex_file
                port_lock_file = self.__shm_dir() / file
                port_segment_file = self.__shm_dir() / file[:-3]
                port_mutex_file = self.__shm_dir() / self.__port_mutex_name(
                    file[:-3])
                self.__remove_file(port_segment_file)
                self.__remove_file(port_lock_file)
                self.__remove_file(port_mutex_file)
                zombie_files += [
                    port_lock_file, port_segment_file, port_mutex_file]
            else:
                self.__ports_in_use += 1

        return [self.__shm_dir() / file_name for file_name in zombie_files]

    def __port_mutex_name(self, port_file_name):
        """
        Return the mutex object filename for the given port.

        param port_file_name str:
            port segment file_name, the mutex name is deduced from it.

        returns str:
            mutex file name

        """
        if os.name == 'posix':
            return ''.join(['sem.', port_file_name, '_mutex'])
        else:
            return ''.join([port_file_name, '_mutex'])

    def __remove_file(self, file):
        """
        Delete a file.

        Always return void, even if the function fails.

        param file str: 
            The complete file_path

        """
        try:
            os.remove(file)
        except BaseException:
            pass

    def __is_file_locked(self, file):
        """Return whether a file is locked or not.

        :param file: str with the complete file_path
        :return: bool

        """
        try:
            with open(file, 'ab') as fd:
                if os.name == 'posix':
                    # Lock file in Exclusive mode. Fail if the file is locked.
                    fcntl.flock(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
                elif os.name == 'nt':
                    # Get WIN32 handle to the file
                    h_file = win32file._get_osfhandle(fd.fileno())
                    # Lock file in Exclusive mode. Fail if the file is locked.
                    mode = win32con.LOCKFILE_EXCLUSIVE_LOCK | msvcrt.LK_NBLCK
                    overlapped = pywintypes.OVERLAPPED()
                    win32file.LockFileEx(h_file, mode, 0, -0x10000, overlapped)
            return False
        except BaseException:
            return True
